package cn.iocoder.yudao.module.dms.controller.admin.articlecatogry;

import cn.iocoder.yudao.framework.apilog.core.annotation.ApiAccessLog;
import cn.iocoder.yudao.framework.common.pojo.CommonResult;
import cn.iocoder.yudao.framework.common.util.object.BeanUtils;
import cn.iocoder.yudao.framework.excel.core.util.ExcelUtils;
import cn.iocoder.yudao.module.dms.controller.admin.articlecatogry.vo.ArticleCatogryListReqVO;
import cn.iocoder.yudao.module.dms.controller.admin.articlecatogry.vo.ArticleCatogryRespVO;
import cn.iocoder.yudao.module.dms.controller.admin.articlecatogry.vo.ArticleCatogrySaveReqVO;
import cn.iocoder.yudao.module.dms.dal.dataobject.articlecatogry.ArticleCatogryDO;
import cn.iocoder.yudao.module.dms.service.articlecatogry.ArticleCatogryService;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;
import java.io.IOException;
import java.util.List;

import static cn.iocoder.yudao.framework.apilog.core.enums.OperateTypeEnum.EXPORT;
import static cn.iocoder.yudao.framework.common.pojo.CommonResult.success;

@Tag(name = "管理后台 - 文章分类")
@RestController
@RequestMapping("/dms/article-catogry")
@Validated
public class ArticleCatogryController {

    @Resource
    private ArticleCatogryService articleCatogryService;

    @PostMapping("/create")
    @Operation(summary = "创建文章分类")
    @PreAuthorize("@ss.hasPermission('dms:article-catogry:create')")
    public CommonResult<Long> createArticleCatogry(@Valid @RequestBody ArticleCatogrySaveReqVO createReqVO) {
        return success(articleCatogryService.createArticleCatogry(createReqVO));
    }

    @PutMapping("/update")
    @Operation(summary = "更新文章分类")
    @PreAuthorize("@ss.hasPermission('dms:article-catogry:update')")
    public CommonResult<Boolean> updateArticleCatogry(@Valid @RequestBody ArticleCatogrySaveReqVO updateReqVO) {
        articleCatogryService.updateArticleCatogry(updateReqVO);
        return success(true);
    }

    @DeleteMapping("/delete")
    @Operation(summary = "删除文章分类")
    @Parameter(name = "id", description = "编号", required = true)
    @PreAuthorize("@ss.hasPermission('dms:article-catogry:delete')")
    public CommonResult<Boolean> deleteArticleCatogry(@RequestParam("id") Long id) {
        articleCatogryService.deleteArticleCatogry(id);
        return success(true);
    }

    @GetMapping("/get")
    @Operation(summary = "获得文章分类")
    @Parameter(name = "id", description = "编号", required = true, example = "1024")
    @PreAuthorize("@ss.hasPermission('dms:article-catogry:query')")
    public CommonResult<ArticleCatogryRespVO> getArticleCatogry(@RequestParam("id") Long id) {
        ArticleCatogryDO articleCatogry = articleCatogryService.getArticleCatogry(id);
        return success(BeanUtils.toBean(articleCatogry, ArticleCatogryRespVO.class));
    }

    @GetMapping("/list")
    @Operation(summary = "获得文章分类列表")
    @PreAuthorize("@ss.hasPermission('dms:article-catogry:query')")
    public CommonResult<List<ArticleCatogryRespVO>> getArticleCatogryList(@Valid ArticleCatogryListReqVO listReqVO) {
        List<ArticleCatogryDO> list = articleCatogryService.getArticleCatogryList(listReqVO);
        return success(BeanUtils.toBean(list, ArticleCatogryRespVO.class));
    }

    @GetMapping("/export-excel")
    @Operation(summary = "导出文章分类 Excel")
    @PreAuthorize("@ss.hasPermission('dms:article-catogry:export')")
    @ApiAccessLog(operateType = EXPORT)
    public void exportArticleCatogryExcel(@Valid ArticleCatogryListReqVO listReqVO,
                                          HttpServletResponse response) throws IOException {
        List<ArticleCatogryDO> list = articleCatogryService.getArticleCatogryList(listReqVO);
        // 导出 Excel
        ExcelUtils.write(response, "文章分类.xls", "数据", ArticleCatogryRespVO.class,
                BeanUtils.toBean(list, ArticleCatogryRespVO.class));
    }

}
