package cn.iocoder.yudao.module.dms.controller.admin.checkplan;

import cn.iocoder.yudao.framework.apilog.core.annotation.ApiAccessLog;
import cn.iocoder.yudao.framework.common.pojo.CommonResult;
import cn.iocoder.yudao.framework.common.pojo.PageParam;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.common.util.object.BeanUtils;
import cn.iocoder.yudao.framework.excel.core.util.ExcelUtils;
import cn.iocoder.yudao.module.dms.controller.admin.checkplan.vo.DmsCheckPlanPageReqVO;
import cn.iocoder.yudao.module.dms.controller.admin.checkplan.vo.DmsCheckPlanRespVO;
import cn.iocoder.yudao.module.dms.controller.admin.checkplan.vo.DmsCheckPlanSaveReqVO;
import cn.iocoder.yudao.module.dms.dal.dataobject.checkplan.DmsCheckPlanDO;
import cn.iocoder.yudao.module.dms.service.checkplan.DmsCheckPlanService;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;
import java.io.IOException;
import java.util.List;

import static cn.iocoder.yudao.framework.apilog.core.enums.OperateTypeEnum.EXPORT;
import static cn.iocoder.yudao.framework.common.pojo.CommonResult.success;

@Tag(name = "管理后台 - 点巡检计划")
@RestController
@RequestMapping("/dms/check-plan")
@Validated
public class DmsCheckPlanController {

    @Resource
    private DmsCheckPlanService checkPlanService;

    @PostMapping("/create")
    @Operation(summary = "创建点巡检计划")
    @PreAuthorize("@ss.hasPermission('dms:check-plan:create')")
    public CommonResult<Long> createCheckPlan(@Valid @RequestBody DmsCheckPlanSaveReqVO createReqVO) {
        return success(checkPlanService.createCheckPlan(createReqVO));
    }

    @PutMapping("/update")
    @Operation(summary = "更新点巡检计划")
    @PreAuthorize("@ss.hasPermission('dms:check-plan:update')")
    public CommonResult<Boolean> updateCheckPlan(@Valid @RequestBody DmsCheckPlanSaveReqVO updateReqVO) {
        checkPlanService.updateCheckPlan(updateReqVO);
        return success(true);
    }

    @DeleteMapping("/delete")
    @Operation(summary = "删除点巡检计划")
    @Parameter(name = "id", description = "编号", required = true)
    @PreAuthorize("@ss.hasPermission('dms:check-plan:delete')")
    public CommonResult<Boolean> deleteCheckPlan(@RequestParam("id") Long id) {
        checkPlanService.deleteCheckPlan(id);
        return success(true);
    }

    @GetMapping("/get")
    @Operation(summary = "获得点巡检计划")
    @Parameter(name = "id", description = "编号", required = true, example = "1024")
    @PreAuthorize("@ss.hasPermission('dms:check-plan:query')")
    public CommonResult<DmsCheckPlanRespVO> getCheckPlan(@RequestParam("id") Long id) {
        DmsCheckPlanDO checkPlan = checkPlanService.getCheckPlan(id);
        return success(BeanUtils.toBean(checkPlan, DmsCheckPlanRespVO.class));
    }

    @GetMapping("/page")
    @Operation(summary = "获得点巡检计划分页")
    @PreAuthorize("@ss.hasPermission('dms:check-plan:query')")
    public CommonResult<PageResult<DmsCheckPlanRespVO>> getCheckPlanPage(@Valid DmsCheckPlanPageReqVO pageReqVO) {
        PageResult<DmsCheckPlanDO> pageResult = checkPlanService.getCheckPlanPage(pageReqVO);
        return success(BeanUtils.toBean(pageResult, DmsCheckPlanRespVO.class));
    }

    @GetMapping("/export-excel")
    @Operation(summary = "导出点巡检计划 Excel")
    @PreAuthorize("@ss.hasPermission('dms:check-plan:export')")
    @ApiAccessLog(operateType = EXPORT)
    public void exportCheckPlanExcel(@Valid DmsCheckPlanPageReqVO pageReqVO,
                                     HttpServletResponse response) throws IOException {
        pageReqVO.setPageSize(PageParam.PAGE_SIZE_NONE);
        List<DmsCheckPlanDO> list = checkPlanService.getCheckPlanPage(pageReqVO).getList();
        // 导出 Excel
        ExcelUtils.write(response, "点巡检计划.xls", "数据", DmsCheckPlanRespVO.class,
                BeanUtils.toBean(list, DmsCheckPlanRespVO.class));
    }

}
