package cn.iocoder.yudao.module.dms.controller.admin.checktask;

import cn.iocoder.yudao.framework.apilog.core.annotation.ApiAccessLog;
import cn.iocoder.yudao.framework.common.pojo.CommonResult;
import cn.iocoder.yudao.framework.common.pojo.PageParam;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.common.util.object.BeanUtils;
import cn.iocoder.yudao.framework.excel.core.util.ExcelUtils;
import cn.iocoder.yudao.module.dms.controller.admin.checktask.vo.DmsCheckTaskPageReqVO;
import cn.iocoder.yudao.module.dms.controller.admin.checktask.vo.DmsCheckTaskRespVO;
import cn.iocoder.yudao.module.dms.controller.admin.checktask.vo.DmsCheckTaskSaveReqVO;
import cn.iocoder.yudao.module.dms.dal.dataobject.checktask.DmsCheckTaskDO;
import cn.iocoder.yudao.module.dms.service.checktask.DmsCheckTaskService;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;
import java.io.IOException;
import java.util.List;
import java.util.Map;

import static cn.iocoder.yudao.framework.apilog.core.enums.OperateTypeEnum.EXPORT;
import static cn.iocoder.yudao.framework.common.pojo.CommonResult.success;

@Tag(name = "管理后台 - 点巡检任务")
@RestController
@RequestMapping("/dms/check-task")
@Validated
public class DmsCheckTaskController {

    @Resource
    private DmsCheckTaskService checkTaskService;

    @PostMapping("/create")
    @Operation(summary = "创建点巡检任务")
    @PreAuthorize("@ss.hasPermission('dms:check-task:create')")
    public CommonResult<Long> createCheckTask(@Valid @RequestBody DmsCheckTaskSaveReqVO createReqVO) {
        return success(checkTaskService.createCheckTask(createReqVO));
    }

    @PutMapping("/update")
    @Operation(summary = "更新点巡检任务")
    @PreAuthorize("@ss.hasPermission('dms:check-task:update')")
    public CommonResult<Boolean> updateCheckTask(@Valid @RequestBody DmsCheckTaskSaveReqVO updateReqVO) {
        checkTaskService.updateCheckTask(updateReqVO);
        return success(true);
    }

    @DeleteMapping("/delete")
    @Operation(summary = "删除点巡检任务")
    @Parameter(name = "id", description = "编号", required = true)
    @PreAuthorize("@ss.hasPermission('dms:check-task:delete')")
    public CommonResult<Boolean> deleteCheckTask(@RequestParam("id") Long id) {
        checkTaskService.deleteCheckTask(id);
        return success(true);
    }

    @GetMapping("/get")
    @Operation(summary = "获得点巡检任务")
    @Parameter(name = "id", description = "编号", required = true, example = "1024")
    @PreAuthorize("@ss.hasPermission('dms:check-task:query')")
    public CommonResult<DmsCheckTaskRespVO> getCheckTask(@RequestParam("id") Long id) {
        DmsCheckTaskDO checkTask = checkTaskService.getCheckTask(id);
        return success(BeanUtils.toBean(checkTask, DmsCheckTaskRespVO.class));
    }

    @GetMapping("/page")
    @Operation(summary = "获得点巡检任务分页")
    @PreAuthorize("@ss.hasPermission('dms:check-task:query')")
    public CommonResult<PageResult<DmsCheckTaskRespVO>> getCheckTaskPage(@Valid DmsCheckTaskPageReqVO pageReqVO) {
        PageResult<DmsCheckTaskDO> pageResult = checkTaskService.getCheckTaskPage(pageReqVO);
        return success(BeanUtils.toBean(pageResult, DmsCheckTaskRespVO.class));
    }


    @PostMapping("/postPage")
    @Operation(summary = "获得点巡检任务分页")
    @PreAuthorize("@ss.hasPermission('dms:check-task:query')")
    public CommonResult<PageResult<DmsCheckTaskRespVO>> postPage(@RequestBody DmsCheckTaskPageReqVO pageReqVO) {
        PageResult<DmsCheckTaskDO> pageResult = checkTaskService.getCheckTaskPage(pageReqVO);
        return success(BeanUtils.toBean(pageResult, DmsCheckTaskRespVO.class));
    }


    @GetMapping("/export-excel")
    @Operation(summary = "导出点巡检任务 Excel")
    @PreAuthorize("@ss.hasPermission('dms:check-task:export')")
    @ApiAccessLog(operateType = EXPORT)
    public void exportCheckTaskExcel(@Valid DmsCheckTaskPageReqVO pageReqVO,
                                     HttpServletResponse response) throws IOException {
        pageReqVO.setPageSize(PageParam.PAGE_SIZE_NONE);
        List<DmsCheckTaskDO> list = checkTaskService.getCheckTaskPage(pageReqVO).getList();
        // 导出 Excel
        ExcelUtils.write(response, "点巡检任务.xls", "数据", DmsCheckTaskRespVO.class,
                BeanUtils.toBean(list, DmsCheckTaskRespVO.class));
    }

    @GetMapping("/tasks")
    @Operation(summary = "统计任务数量")
    @PreAuthorize("@ss.hasPermission('dms:check-task:query')")
    public CommonResult<Map<String, Object>> tasks() {
        return success(checkTaskService.tasks());
    }

    @PostMapping("/countCur")
    @Operation(summary = "统计未完成任务数量")
    @PreAuthorize("@ss.hasPermission('dms:check-task:query')")
    public CommonResult<Long> countCur(@RequestBody DmsCheckTaskDO checkTaskDO) {
        return success(checkTaskService.countCur(checkTaskDO));
    }


    @PutMapping("/tmpSave")
    @Operation(summary = "任务暂存")
    @PreAuthorize("@ss.hasPermission('dms:check-task:update')")
    public CommonResult<String> tmpSave(@Valid @RequestBody DmsCheckTaskDO checkTaskDO) {
        return success(checkTaskService.tmpSave(checkTaskDO));
    }

    @PutMapping("/commit")
    @Operation(summary = "任务提交")
    @PreAuthorize("@ss.hasPermission('dms:check-task:update')")
    public CommonResult<String> commit(@Valid @RequestBody DmsCheckTaskDO checkTaskDO) {
        return success(checkTaskService.commit(checkTaskDO));
    }

    @PostMapping("/getFirst")
    @Operation(summary = "获得点巡检任务分页")
    @PreAuthorize("@ss.hasPermission('dms:check-task:query')")
    public CommonResult<DmsCheckTaskRespVO> getFirst(@RequestBody DmsCheckTaskPageReqVO pageReqVO) {
        DmsCheckTaskDO res = checkTaskService.getFirst(pageReqVO);
        return success(BeanUtils.toBean(res, DmsCheckTaskRespVO.class));
    }
}
