package cn.iocoder.yudao.module.dms.controller.admin.device;

import cn.iocoder.yudao.framework.apilog.core.annotation.ApiAccessLog;
import cn.iocoder.yudao.framework.common.pojo.CommonResult;
import cn.iocoder.yudao.framework.common.pojo.PageParam;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.common.util.object.BeanUtils;
import cn.iocoder.yudao.framework.excel.core.util.ExcelUtils;
import cn.iocoder.yudao.module.dms.controller.admin.device.vo.DmsDevicePageReqVO;
import cn.iocoder.yudao.module.dms.controller.admin.device.vo.DmsDeviceRespVO;
import cn.iocoder.yudao.module.dms.controller.admin.device.vo.DmsDeviceSaveReqVO;
import cn.iocoder.yudao.module.dms.dal.dataobject.device.DmsDeviceDO;
import cn.iocoder.yudao.module.dms.service.device.DmsDeviceService;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;
import java.io.IOException;
import java.util.List;

import static cn.iocoder.yudao.framework.apilog.core.enums.OperateTypeEnum.EXPORT;
import static cn.iocoder.yudao.framework.common.pojo.CommonResult.success;

@Tag(name = "管理后台 - 设备管理")
@RestController
@RequestMapping("/dms/device")
@Validated
public class DmsDeviceController {

    @Resource
    private DmsDeviceService deviceService;

    @PostMapping("/create")
    @Operation(summary = "创建设备管理")
    @PreAuthorize("@ss.hasPermission('dms:device:create')")
    public CommonResult<Long> createDevice(@Valid @RequestBody DmsDeviceSaveReqVO createReqVO) {
        return success(deviceService.createDevice(createReqVO));
    }

    @PutMapping("/update")
    @Operation(summary = "更新设备管理")
    @PreAuthorize("@ss.hasPermission('dms:device:update')")
    public CommonResult<Boolean> updateDevice(@Valid @RequestBody DmsDeviceSaveReqVO updateReqVO) {
        deviceService.updateDevice(updateReqVO);
        return success(true);
    }

    @DeleteMapping("/delete")
    @Operation(summary = "删除设备管理")
    @Parameter(name = "id", description = "编号", required = true)
    @PreAuthorize("@ss.hasPermission('dms:device:delete')")
    public CommonResult<Boolean> deleteDevice(@RequestParam("id") Long id) {
        deviceService.deleteDevice(id);
        return success(true);
    }

    @GetMapping("/get")
    @Operation(summary = "获得设备管理")
    @Parameter(name = "id", description = "编号", required = true, example = "1024")
    @PreAuthorize("@ss.hasPermission('dms:device:query')")
    public CommonResult<DmsDeviceRespVO> getDevice(@RequestParam("id") Long id) {
        DmsDeviceDO device = deviceService.getDevice(id);
        return success(BeanUtils.toBean(device, DmsDeviceRespVO.class));
    }

    @GetMapping("/page")
    @Operation(summary = "获得设备管理分页")
    @PreAuthorize("@ss.hasPermission('dms:device:query')")
    public CommonResult<PageResult<DmsDeviceRespVO>> getDevicePage(@Valid DmsDevicePageReqVO pageReqVO) {
        PageResult<DmsDeviceDO> pageResult = deviceService.getDevicePage(pageReqVO);
        return success(BeanUtils.toBean(pageResult, DmsDeviceRespVO.class));
    }

    @GetMapping("/export-excel")
    @Operation(summary = "导出设备管理 Excel")
    @PreAuthorize("@ss.hasPermission('dms:device:export')")
    @ApiAccessLog(operateType = EXPORT)
    public void exportDeviceExcel(@Valid DmsDevicePageReqVO pageReqVO,
                                  HttpServletResponse response) throws IOException {
        pageReqVO.setPageSize(PageParam.PAGE_SIZE_NONE);
        List<DmsDeviceDO> list = deviceService.getDevicePage(pageReqVO).getList();
        // 导出 Excel
        ExcelUtils.write(response, "设备管理.xls", "数据", DmsDeviceRespVO.class,
                BeanUtils.toBean(list, DmsDeviceRespVO.class));
    }

    @PostMapping("/location")
    @Operation(summary = "设备定位")
    @PreAuthorize("@ss.hasPermission('dms:device:update')")
    public CommonResult<Long> location(@RequestBody DmsDeviceSaveReqVO createReqVO) {
        return success(deviceService.location(createReqVO));
    }

    @PostMapping("/locationCount")
    @Operation(summary = "设备未定位数量统计")
    @PreAuthorize("@ss.hasPermission('dms:device:query')")
    public CommonResult<Long> locationCount(@RequestBody DmsDeviceSaveReqVO saveReqVO) {
        return success(deviceService.locationCount(saveReqVO));
    }

}
