package cn.iocoder.yudao.module.dms.controller.admin.deviceclassify;

import cn.iocoder.yudao.framework.apilog.core.annotation.ApiAccessLog;
import cn.iocoder.yudao.framework.common.pojo.CommonResult;
import cn.iocoder.yudao.framework.common.util.object.BeanUtils;
import cn.iocoder.yudao.framework.excel.core.util.ExcelUtils;
import cn.iocoder.yudao.module.dms.controller.admin.deviceclassify.vo.DmsDeviceClassifyListReqVO;
import cn.iocoder.yudao.module.dms.controller.admin.deviceclassify.vo.DmsDeviceClassifyRespVO;
import cn.iocoder.yudao.module.dms.controller.admin.deviceclassify.vo.DmsDeviceClassifySaveReqVO;
import cn.iocoder.yudao.module.dms.controller.admin.deviceclassify.vo.DmsSimpleDeviceClassifyRespVO;
import cn.iocoder.yudao.module.dms.dal.dataobject.deviceclassify.DmsDeviceClassifyDO;
import cn.iocoder.yudao.module.dms.service.deviceclassify.DmsDeviceClassifyService;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;
import java.io.IOException;
import java.util.List;

import static cn.iocoder.yudao.framework.apilog.core.enums.OperateTypeEnum.EXPORT;
import static cn.iocoder.yudao.framework.common.pojo.CommonResult.success;

@Tag(name = "管理后台 - 设备分类")
@RestController
@RequestMapping("/dms/device-classify")
@Validated
public class DmsDeviceClassifyController {

    @Resource
    private DmsDeviceClassifyService deviceClassifyService;

    @PostMapping("/create")
    @Operation(summary = "创建设备分类")
    @PreAuthorize("@ss.hasPermission('dms:device-classify:create')")
    public CommonResult<Long> createDeviceClassify(@Valid @RequestBody DmsDeviceClassifySaveReqVO createReqVO) {
        return success(deviceClassifyService.createDeviceClassify(createReqVO));
    }

    @PutMapping("/update")
    @Operation(summary = "更新设备分类")
    @PreAuthorize("@ss.hasPermission('dms:device-classify:update')")
    public CommonResult<Boolean> updateDeviceClassify(@Valid @RequestBody DmsDeviceClassifySaveReqVO updateReqVO) {
        deviceClassifyService.updateDeviceClassify(updateReqVO);
        return success(true);
    }

    @DeleteMapping("/delete")
    @Operation(summary = "删除设备分类")
    @Parameter(name = "id", description = "编号", required = true)
    @PreAuthorize("@ss.hasPermission('dms:device-classify:delete')")
    public CommonResult<Boolean> deleteDeviceClassify(@RequestParam("id") Long id) {
        deviceClassifyService.deleteDeviceClassify(id);
        return success(true);
    }

    @GetMapping("/get")
    @Operation(summary = "获得设备分类")
    @Parameter(name = "id", description = "编号", required = true, example = "1024")
    @PreAuthorize("@ss.hasPermission('dms:device-classify:query')")
    public CommonResult<DmsDeviceClassifyRespVO> getDeviceClassify(@RequestParam("id") Long id) {
        DmsDeviceClassifyDO deviceClassify = deviceClassifyService.getDeviceClassify(id);
        return success(BeanUtils.toBean(deviceClassify, DmsDeviceClassifyRespVO.class));
    }

    @GetMapping("/list")
    @Operation(summary = "获得设备分类列表")
    @PreAuthorize("@ss.hasPermission('dms:device-classify:query')")
    public CommonResult<List<DmsDeviceClassifyRespVO>> getDeviceClassifyList(@Valid DmsDeviceClassifyListReqVO listReqVO) {
        List<DmsDeviceClassifyDO> list = deviceClassifyService.getDeviceClassifyList(listReqVO);
        return success(BeanUtils.toBean(list, DmsDeviceClassifyRespVO.class));
    }

    @GetMapping("/simple-list")
    @Operation(summary = "获得设备分类精简列表", description = "主要用于前端的下拉选项")
    public CommonResult<List<DmsSimpleDeviceClassifyRespVO>> getSimpleDeviceClassifyList() {
        List<DmsDeviceClassifyDO> list = deviceClassifyService.getSimpleDeviceClassifyList();
        return success(BeanUtils.toBean(list, DmsSimpleDeviceClassifyRespVO.class));
    }

    @GetMapping("/export-excel")
    @Operation(summary = "导出设备分类 Excel")
    @PreAuthorize("@ss.hasPermission('dms:device-classify:export')")
    @ApiAccessLog(operateType = EXPORT)
    public void exportDeviceClassifyExcel(@Valid DmsDeviceClassifyListReqVO listReqVO,
                                          HttpServletResponse response) throws IOException {
        List<DmsDeviceClassifyDO> list = deviceClassifyService.getDeviceClassifyList(listReqVO);
        // 导出 Excel
        ExcelUtils.write(response, "设备分类.xls", "数据", DmsDeviceClassifyRespVO.class,
                BeanUtils.toBean(list, DmsDeviceClassifyRespVO.class));
    }

}
