package cn.iocoder.yudao.module.dms.controller.admin.relationorg;

import cn.iocoder.yudao.framework.apilog.core.annotation.ApiAccessLog;
import cn.iocoder.yudao.framework.common.pojo.CommonResult;
import cn.iocoder.yudao.framework.common.pojo.PageParam;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.common.util.object.BeanUtils;
import cn.iocoder.yudao.framework.excel.core.util.ExcelUtils;
import cn.iocoder.yudao.module.dms.controller.admin.relationorg.vo.DmsRelationOrgPageReqVO;
import cn.iocoder.yudao.module.dms.controller.admin.relationorg.vo.DmsRelationOrgRespVO;
import cn.iocoder.yudao.module.dms.controller.admin.relationorg.vo.DmsRelationOrgSaveReqVO;
import cn.iocoder.yudao.module.dms.controller.admin.relationorg.vo.DmsSimpleRelationOrgRespVO;
import cn.iocoder.yudao.module.dms.dal.dataobject.relationorg.DmsRelationContractDO;
import cn.iocoder.yudao.module.dms.dal.dataobject.relationorg.DmsRelationOrgDO;
import cn.iocoder.yudao.module.dms.service.relationorg.DmsRelationOrgService;
import io.swagger.v3.oas.annotations.Operation;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.tags.Tag;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import javax.validation.Valid;
import java.io.IOException;
import java.util.List;

import static cn.iocoder.yudao.framework.apilog.core.enums.OperateTypeEnum.EXPORT;
import static cn.iocoder.yudao.framework.common.pojo.CommonResult.success;

@Tag(name = "管理后台 - 往来单位")
@RestController
@RequestMapping("/dms/relation-org")
@Validated
public class DmsRelationOrgController {

    @Resource
    private DmsRelationOrgService relationOrgService;

    @PostMapping("/create")
    @Operation(summary = "创建往来单位")
    @PreAuthorize("@ss.hasPermission('dms:relation-org:create')")
    public CommonResult<Long> createRelationOrg(@Valid @RequestBody DmsRelationOrgSaveReqVO createReqVO) {
        return success(relationOrgService.createRelationOrg(createReqVO));
    }

    @PutMapping("/update")
    @Operation(summary = "更新往来单位")
    @PreAuthorize("@ss.hasPermission('dms:relation-org:update')")
    public CommonResult<Boolean> updateRelationOrg(@Valid @RequestBody DmsRelationOrgSaveReqVO updateReqVO) {
        relationOrgService.updateRelationOrg(updateReqVO);
        return success(true);
    }

    @DeleteMapping("/delete")
    @Operation(summary = "删除往来单位")
    @Parameter(name = "id", description = "编号", required = true)
    @PreAuthorize("@ss.hasPermission('dms:relation-org:delete')")
    public CommonResult<Boolean> deleteRelationOrg(@RequestParam("id") Long id) {
        relationOrgService.deleteRelationOrg(id);
        return success(true);
    }

    @GetMapping("/get")
    @Operation(summary = "获得往来单位")
    @Parameter(name = "id", description = "编号", required = true, example = "1024")
    @PreAuthorize("@ss.hasPermission('dms:relation-org:query')")
    public CommonResult<DmsRelationOrgRespVO> getRelationOrg(@RequestParam("id") Long id) {
        DmsRelationOrgDO relationOrg = relationOrgService.getRelationOrg(id);
        return success(BeanUtils.toBean(relationOrg, DmsRelationOrgRespVO.class));
    }

    @GetMapping("/page")
    @Operation(summary = "获得往来单位分页")
    @PreAuthorize("@ss.hasPermission('dms:relation-org:query')")
    public CommonResult<PageResult<DmsRelationOrgRespVO>> getRelationOrgPage(@Valid DmsRelationOrgPageReqVO pageReqVO) {
        PageResult<DmsRelationOrgDO> pageResult = relationOrgService.getRelationOrgPage(pageReqVO);
        return success(BeanUtils.toBean(pageResult, DmsRelationOrgRespVO.class));
    }

    @GetMapping("/simple-list")
    @Operation(summary = "获得往来单位精简列表", description = "主要用于前端的下拉选项")
    public CommonResult<List<DmsSimpleRelationOrgRespVO>> getSimpleDeviceClassifyList() {
        List<DmsRelationOrgDO> list = relationOrgService.getSimpleRelationOrgList();
        return success(BeanUtils.toBean(list, DmsSimpleRelationOrgRespVO.class));
    }

    @GetMapping("/export-excel")
    @Operation(summary = "导出往来单位 Excel")
    @PreAuthorize("@ss.hasPermission('dms:relation-org:export')")
    @ApiAccessLog(operateType = EXPORT)
    public void exportRelationOrgExcel(@Valid DmsRelationOrgPageReqVO pageReqVO,
                                       HttpServletResponse response) throws IOException {
        pageReqVO.setPageSize(PageParam.PAGE_SIZE_NONE);
        List<DmsRelationOrgDO> list = relationOrgService.getRelationOrgPage(pageReqVO).getList();
        // 导出 Excel
        ExcelUtils.write(response, "往来单位.xls", "数据", DmsRelationOrgRespVO.class,
                BeanUtils.toBean(list, DmsRelationOrgRespVO.class));
    }

    // ==================== 子表（往来单位联系人） ====================

    @GetMapping("/relation-contract/page")
    @Operation(summary = "获得往来单位联系人分页")
    @Parameter(name = "orgId", description = "来往单位ID")
    @PreAuthorize("@ss.hasPermission('dms:relation-org:query')")
    public CommonResult<PageResult<DmsRelationContractDO>> getRelationContractPage(PageParam pageReqVO,
                                                                                   @RequestParam("orgId") Long orgId) {
        return success(relationOrgService.getRelationContractPage(pageReqVO, orgId));
    }

    @PostMapping("/relation-contract/create")
    @Operation(summary = "创建往来单位联系人")
    @PreAuthorize("@ss.hasPermission('dms:relation-org:create')")
    public CommonResult<Long> createRelationContract(@Valid @RequestBody DmsRelationContractDO relationContract) {
        return success(relationOrgService.createRelationContract(relationContract));
    }

    @PutMapping("/relation-contract/update")
    @Operation(summary = "更新往来单位联系人")
    @PreAuthorize("@ss.hasPermission('dms:relation-org:update')")
    public CommonResult<Boolean> updateRelationContract(@Valid @RequestBody DmsRelationContractDO relationContract) {
        relationOrgService.updateRelationContract(relationContract);
        return success(true);
    }

    @DeleteMapping("/relation-contract/delete")
    @Parameter(name = "id", description = "编号", required = true)
    @Operation(summary = "删除往来单位联系人")
    @PreAuthorize("@ss.hasPermission('dms:relation-org:delete')")
    public CommonResult<Boolean> deleteRelationContract(@RequestParam("id") Long id) {
        relationOrgService.deleteRelationContract(id);
        return success(true);
    }

    @GetMapping("/relation-contract/get")
    @Operation(summary = "获得往来单位联系人")
    @Parameter(name = "id", description = "编号", required = true)
    @PreAuthorize("@ss.hasPermission('dms:relation-org:query')")
    public CommonResult<DmsRelationContractDO> getRelationContract(@RequestParam("id") Long id) {
        return success(relationOrgService.getRelationContract(id));
    }

}
