package cn.iocoder.yudao.module.dms.service.checkplan;

import cn.iocoder.yudao.framework.common.enums.CommonStatusEnum;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.common.util.object.BeanUtils;
import cn.iocoder.yudao.framework.security.core.LoginUser;
import cn.iocoder.yudao.framework.security.core.util.SecurityFrameworkUtils;
import cn.iocoder.yudao.framework.tenant.core.aop.TenantIgnore;
import cn.iocoder.yudao.framework.tenant.core.util.TenantUtils;
import cn.iocoder.yudao.module.dms.controller.admin.checkplan.vo.DmsCheckPlanPageReqVO;
import cn.iocoder.yudao.module.dms.controller.admin.checkplan.vo.DmsCheckPlanSaveReqVO;
import cn.iocoder.yudao.module.dms.dal.dataobject.checkplan.DmsCheckPlanDO;
import cn.iocoder.yudao.module.dms.dal.dataobject.checktask.DmsCheckTaskDO;
import cn.iocoder.yudao.module.dms.dal.dataobject.checktaskitem.DmsCheckTaskItemDO;
import cn.iocoder.yudao.module.dms.dal.dataobject.checktemplateitem.DmsCheckTemplateItemDO;
import cn.iocoder.yudao.module.dms.dal.dataobject.device.DmsDeviceDO;
import cn.iocoder.yudao.module.dms.dal.pgsql.checkplan.DmsCheckPlanMapper;
import cn.iocoder.yudao.module.dms.dal.pgsql.checktask.DmsCheckTaskMapper;
import cn.iocoder.yudao.module.dms.dal.pgsql.checktemplateitem.DmsCheckTemplateItemMapper;
import cn.iocoder.yudao.module.dms.dal.pgsql.device.DmsDeviceMapper;
import cn.iocoder.yudao.module.dms.service.checktaskitem.DmsCheckTaskItemService;
import cn.iocoder.yudao.module.system.api.dict.DictDataApi;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Service;
import org.springframework.validation.annotation.Validated;

import javax.annotation.Resource;
import java.time.LocalDateTime;
import java.util.List;
import java.util.Objects;
import java.util.stream.Collectors;

import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.dms.enums.ErrorCodeConstants.CHECK_PLAN_NOT_EXISTS;

/**
 * 点巡检计划 Service 实现类
 *
 * @author dzx
 */
@Service
@Validated
public class DmsCheckPlanServiceImpl implements DmsCheckPlanService {

    @Resource
    private DmsCheckPlanMapper checkPlanMapper;
    @Resource
    private DmsCheckTaskMapper checkTaskMapper;
    @Resource
    private DmsDeviceMapper dmsDeviceMapper;
    @Resource
    private DictDataApi dictDataApi;
    @Resource
    private DmsCheckTemplateItemMapper dmsCheckTemplateItemMapper;
    @Resource
    private DmsCheckTaskItemService dmsCheckTaskItemService;


    @Override

    public Long createCheckPlan(DmsCheckPlanSaveReqVO createReqVO) {
        // 插入
        DmsCheckPlanDO checkPlan = BeanUtils.toBean(createReqVO, DmsCheckPlanDO.class);
        checkPlan.setDeptId(SecurityFrameworkUtils.getLoginUserDeptId());
        DmsDeviceDO dmsDeviceDO = this.dmsDeviceMapper.selectById(createReqVO.getDeviceId());
        LoginUser loginUser = SecurityFrameworkUtils.getLoginUser();
        checkPlan.setDeviceCode(dmsDeviceDO.getDeviceCode()).setDeviceName(dmsDeviceDO.getName())
                .setDeviceClassifyId(dmsDeviceDO.getCid()).setDeviceClassifyName(dmsDeviceDO.getCname())
                .setDeviceClassifyCode(dmsDeviceDO.getClassifyCode()).setCreatorId(loginUser.getId())
                .setCreatorName(loginUser.getInfo().get(LoginUser.INFO_KEY_NICKNAME));
        checkPlanMapper.insert(checkPlan);
        // 返回
        return checkPlan.getId();
    }

    @Override
    public void updateCheckPlan(DmsCheckPlanSaveReqVO updateReqVO) {
        // 校验存在
        validateCheckPlanExists(updateReqVO.getId());
        // 更新
        DmsCheckPlanDO updateObj = BeanUtils.toBean(updateReqVO, DmsCheckPlanDO.class);
        DmsCheckPlanDO dmsCheckPlanDO = this.checkPlanMapper.selectById(updateObj.getId());
        if (!Objects.equals(dmsCheckPlanDO.getDeviceId(), updateObj.getDeviceId())) {
            DmsDeviceDO dmsDeviceDO = this.dmsDeviceMapper.selectById(updateReqVO.getDeviceId());

            updateObj.setDeviceCode(dmsDeviceDO.getDeviceCode()).setDeviceName(dmsDeviceDO.getName())
                    .setDeviceClassifyId(dmsDeviceDO.getCid()).setDeviceClassifyName(dmsDeviceDO.getCname())
                    .setDeviceClassifyCode(dmsDeviceDO.getClassifyCode());
        }
        checkPlanMapper.updateById(updateObj);
    }

    @Override
    public void deleteCheckPlan(Long id) {
        // 校验存在
        validateCheckPlanExists(id);
        // 删除
        checkPlanMapper.deleteById(id);
    }

    private void validateCheckPlanExists(Long id) {
        if (checkPlanMapper.selectById(id) == null) {
            throw exception(CHECK_PLAN_NOT_EXISTS);
        }
    }

    @Override
    public DmsCheckPlanDO getCheckPlan(Long id) {
        return checkPlanMapper.selectById(id);
    }

    @Override
    public PageResult<DmsCheckPlanDO> getCheckPlanPage(DmsCheckPlanPageReqVO pageReqVO) {
        return checkPlanMapper.selectPage(pageReqVO);
    }

    @Scheduled(cron = "0 */1 * * * ?")
    @TenantIgnore
    public void genTasks() {
        System.out.println("Scheduled......");
        LocalDateTime now = LocalDateTime.now();

        //待生成任务的计划列表
        List<DmsCheckPlanDO> checkPlans = this.checkPlanMapper.selectList(new QueryWrapper<DmsCheckPlanDO>().lambda()
                .eq(DmsCheckPlanDO::getUseFlag, CommonStatusEnum.ENABLE.getStatus()).ge(DmsCheckPlanDO::getEndTime, now));
        if (CollectionUtils.isNotEmpty(checkPlans)) {
//            List<Long> ctIds = checkPlans.stream().map(i -> i.getCheckTemplateId()).collect(Collectors.toList());
//            List<CheckTemplate> checkTemplates = this.checkTemplateMapper.selectList(new LambdaQueryWrapper<CheckTemplate>().in(CheckTemplate::getId, ctIds));
//            Map<Long, CheckTemplate> ctMap = checkTemplates.stream().collect(Collectors.toMap(i -> i.getId(), i -> i));
//            Map<Long, List<CheckTemplateItem>> itemMap = Maps.newHashMap();
            checkPlans.forEach(cp -> {

                TenantUtils.execute(cp.getTenantId(), () -> {
                    Long dbTasNum = checkTaskMapper.selectCount(new LambdaQueryWrapper<DmsCheckTaskDO>()
                            .ge(DmsCheckTaskDO::getStartTime, now)
                            .eq(DmsCheckTaskDO::getDeviceId, cp.getDeviceId()).eq(DmsCheckTaskDO::getCheckPlanId, cp.getId()));
                    if (Objects.equals(cp.getTaskNum(), new Integer(0))) {
                        //下次生产任务数量为零 只在任务提前十二个小时生产任务
                        if (now.isAfter(cp.getStartTime()) && Objects.equals(new Integer(0), dbTasNum)) {
                            DmsCheckTaskDO checkTask = cp.buildTask().setStatus(0).setStartTime(LocalDateTime.now());
                            checkTask.setEndTime(checkTask.getStartTime().plusHours(cp.oneTotal() + cp.executeTotal()));
                            checkTaskMapper.insert(checkTask);
                            dmsCheckTaskItemService.saveBatch(genCheckTaskItems(checkTask));
                        }

                    } else if (dbTasNum < cp.getTaskNum()) {
                        LambdaQueryWrapper<DmsCheckTaskDO> checkTaskLambdaQueryWrapper = new LambdaQueryWrapper<DmsCheckTaskDO>()
                                .eq(DmsCheckTaskDO::getUseFlag, 1)
                                .eq(DmsCheckTaskDO::getDeviceId, cp.getDeviceId()).eq(DmsCheckTaskDO::getCheckPlanId, cp.getId())
                                .ge(DmsCheckTaskDO::getStartTime, cp.getStartTime()).orderByDesc(DmsCheckTaskDO::getEndTime);
                        checkTaskLambdaQueryWrapper.last(" LIMIT 1");
                        DmsCheckTaskDO lastCheckTask = checkTaskMapper.selectOne(checkTaskLambdaQueryWrapper);
                        if (Objects.isNull(lastCheckTask)) {
                            //首次生成任务
                            for (int i = 0; i < cp.getTaskNum(); i++) {
                                DmsCheckTaskDO checkTask = cp.buildTask().setStatus(0)
                                        .setStartTime(cp.getStartTime().plusHours((long) i * cp.oneTotal()))
                                        .setEndTime(cp.getStartTime().plusHours((long) i * cp.oneTotal() + cp.executeTotal()));
                                checkTaskMapper.insert(checkTask);
                                dmsCheckTaskItemService.saveBatch(genCheckTaskItems(checkTask));
                            }
                        } else {
                            //非首次生成任务
                            for (int i = 1; i <= cp.getTaskNum() - dbTasNum; i++) {
                                DmsCheckTaskDO checkTask = cp.buildTask().setStatus(0)
                                        .setStartTime(lastCheckTask.getStartTime().plusHours((long) i * cp.oneTotal()))
                                        .setEndTime(lastCheckTask.getStartTime().plusHours((long) i * cp.oneTotal() + cp.executeTotal()));
                                checkTaskMapper.insert(checkTask);
                                dmsCheckTaskItemService.saveBatch(genCheckTaskItems(checkTask));

                            }
                        }
                    }
                });

            });
        }

    }

    private List<DmsCheckTaskItemDO> genCheckTaskItems(DmsCheckTaskDO checkTask) {
        List<DmsCheckTemplateItemDO> checkTemplateItems = this.dmsCheckTemplateItemMapper.selectList(new LambdaQueryWrapper<DmsCheckTemplateItemDO>()
                .eq(DmsCheckTemplateItemDO::getCheckTemplateId, checkTask.getCheckTemplateId()).
                eq(DmsCheckTemplateItemDO::getUseFlag, 1).orderByAsc(DmsCheckTemplateItemDO::getOrderNum));
        LocalDateTime now = LocalDateTime.now();
//        List<DmsCheckTaskItemDO> adds = checkTemplateItems.stream().map(item -> item.buildCheckTaskItem().setCheckTaskId(checkTask.getId()).setCreateTime(now)).collect(Collectors.toList());
        List<DmsCheckTaskItemDO> list =
                checkTemplateItems.stream().map(item -> item.buildCheckTaskItem().setCheckTaskId(checkTask.getId())).collect(Collectors.toList());
        return list;
    }

}
