package cn.iocoder.yudao.module.dms.service.checktask;

import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.common.util.object.BeanUtils;
import cn.iocoder.yudao.framework.security.core.util.SecurityFrameworkUtils;
import cn.iocoder.yudao.module.dms.controller.admin.checktask.vo.DmsCheckTaskPageReqVO;
import cn.iocoder.yudao.module.dms.controller.admin.checktask.vo.DmsCheckTaskSaveReqVO;
import cn.iocoder.yudao.module.dms.dal.dataobject.checktask.DmsCheckTaskDO;
import cn.iocoder.yudao.module.dms.dal.dataobject.checktaskitem.DmsCheckTaskItemDO;
import cn.iocoder.yudao.module.dms.dal.pgsql.checktask.DmsCheckTaskMapper;
import cn.iocoder.yudao.module.dms.dal.pgsql.checktaskitem.DmsCheckTaskItemMapper;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.google.common.collect.Maps;
import org.springframework.stereotype.Service;
import org.springframework.validation.annotation.Validated;

import javax.annotation.Resource;
import java.time.LocalDateTime;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.stream.Collectors;

import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.dms.enums.ErrorCodeConstants.CHECK_TASK_NOT_EXISTS;

/**
 * 点巡检任务 Service 实现类
 *
 * @author dzx
 */
@Service
@Validated
public class DmsCheckTaskServiceImpl implements DmsCheckTaskService {

    @Resource
    private DmsCheckTaskMapper checkTaskMapper;
    @Resource
    private DmsCheckTaskItemMapper checkTaskItemMapper;

    @Override
    public Long createCheckTask(DmsCheckTaskSaveReqVO createReqVO) {
        // 插入
        DmsCheckTaskDO checkTask = BeanUtils.toBean(createReqVO, DmsCheckTaskDO.class);
        checkTask.setDeptId(SecurityFrameworkUtils.getLoginUserDeptId());
        checkTaskMapper.insert(checkTask);
        // 返回
        return checkTask.getId();
    }

    @Override
    public void updateCheckTask(DmsCheckTaskSaveReqVO updateReqVO) {
        // 校验存在
        validateCheckTaskExists(updateReqVO.getId());
        // 更新
        DmsCheckTaskDO updateObj = BeanUtils.toBean(updateReqVO, DmsCheckTaskDO.class);
        checkTaskMapper.updateById(updateObj);
    }

    @Override
    public void deleteCheckTask(Long id) {
        // 校验存在
        validateCheckTaskExists(id);
        // 删除
        checkTaskMapper.deleteById(id);
    }

    private void validateCheckTaskExists(Long id) {
        if (checkTaskMapper.selectById(id) == null) {
            throw exception(CHECK_TASK_NOT_EXISTS);
        }
    }

    @Override
    public DmsCheckTaskDO getCheckTask(Long id) {
        return checkTaskMapper.selectById(id);
    }

    @Override
    public PageResult<DmsCheckTaskDO> getCheckTaskPage(DmsCheckTaskPageReqVO pageReqVO) {
        return checkTaskMapper.selectPage(pageReqVO);
    }


    @Override
    public Map<String, Object> tasks() {
//        User user = this.userMapper.selectById(Convert.toLong(StpUtil.getLoginId()));
        Long userId = SecurityFrameworkUtils.getLoginUserId();
        LocalDateTime now = LocalDateTime.now();
        Long total = this.checkTaskMapper.selectCount(new LambdaQueryWrapper<DmsCheckTaskDO>()
                .eq(DmsCheckTaskDO::getExecutorId, userId).le(DmsCheckTaskDO::getStartTime, now)
                .eq(DmsCheckTaskDO::getUseFlag, 1));
        Long end = this.checkTaskMapper.selectCount(new LambdaQueryWrapper<DmsCheckTaskDO>()
                .eq(DmsCheckTaskDO::getExecutorId, userId).le(DmsCheckTaskDO::getStartTime, now)
                .eq(DmsCheckTaskDO::getUseFlag, 1).eq(DmsCheckTaskDO::getEndFlag, 1));

        List<DmsCheckTaskDO> checkTasks = this.checkTaskMapper.selectList(new LambdaQueryWrapper<DmsCheckTaskDO>()
                .eq(DmsCheckTaskDO::getExecutorId, userId).eq(DmsCheckTaskDO::getUseFlag, 1)
                .eq(DmsCheckTaskDO::getEndFlag, 0).orderByAsc(DmsCheckTaskDO::getStartTime));

        LambdaQueryWrapper endListWrapper = new LambdaQueryWrapper<DmsCheckTaskDO>()
                .eq(DmsCheckTaskDO::getExecutorId, userId).eq(DmsCheckTaskDO::getUseFlag, 1)
                .eq(DmsCheckTaskDO::getEndFlag, 1).orderByDesc(DmsCheckTaskDO::getStartTime);
        endListWrapper.last("limit 10");
        List endList = this.checkTaskMapper.selectList(endListWrapper);

        Map<String, Object> map = Maps.newHashMap();
        map.put("total", total);
        map.put("end", end);
        map.put("nodoTasks", checkTasks.stream().filter(item -> item.getStartTime().isBefore(now)).collect(Collectors.toList()));
        map.put("todoTasks", checkTasks.stream().filter(item -> item.getStartTime().isAfter(now)).collect(Collectors.toList()));
        map.put("endTasks", endList);
        return map;
    }

    @Override
    public String tmpSave(DmsCheckTaskDO checkTask) {
        LocalDateTime now = LocalDateTime.now();
        this.checkTaskMapper.update(null, new LambdaUpdateWrapper<DmsCheckTaskDO>().set(DmsCheckTaskDO::getNote, checkTask.getNote())
                .set(DmsCheckTaskDO::getUpdateTime, now).set(DmsCheckTaskDO::getStatus, 2).
                eq(DmsCheckTaskDO::getId, checkTask.getId()));

        Map<Long, DmsCheckTaskItemDO> itemMap = checkTask.getCheckTaskItems().stream().collect(Collectors.toMap(item -> item.getId(), item -> item));
        List<DmsCheckTaskItemDO> taskItems = this.checkTaskItemMapper.selectList(new LambdaQueryWrapper<DmsCheckTaskItemDO>()
                .eq(DmsCheckTaskItemDO::getCheckTaskId, checkTask.getId()).eq(DmsCheckTaskItemDO::getUseFlag, 1));
        taskItems.stream().forEach(item -> {
            item.setContent(itemMap.get(item.getId()).getContent())
                    .setImgs(itemMap.get(item.getId()).getImgs()).setUpdateTime(now);
        });
        this.checkTaskItemMapper.updateBatch(taskItems);
        return "任务暂存成功!";
    }

    @Override
    public String commit(DmsCheckTaskDO checkTask) {
        LocalDateTime now = LocalDateTime.now();
        this.checkTaskMapper.update(null, new LambdaUpdateWrapper<DmsCheckTaskDO>().set(DmsCheckTaskDO::getNote, checkTask.getNote())
                .set(DmsCheckTaskDO::getStatus, 3)
                .set(DmsCheckTaskDO::getExecutorTime, now).set(DmsCheckTaskDO::getUpdateTime, now)
                .set(DmsCheckTaskDO::getEndFlag, 1).eq(DmsCheckTaskDO::getId, checkTask.getId()));

        Map<Long, DmsCheckTaskItemDO> itemMap = checkTask.getCheckTaskItems().stream().collect(Collectors.toMap(item -> item.getId(), item -> item));
        LambdaQueryWrapper<DmsCheckTaskItemDO> eq = new LambdaQueryWrapper<DmsCheckTaskItemDO>().eq(DmsCheckTaskItemDO::getUseFlag, 1)
                .eq(DmsCheckTaskItemDO::getCheckTaskId, checkTask.getId());
        List<DmsCheckTaskItemDO> taskItems = this.checkTaskItemMapper.selectList(eq);
        taskItems.stream().forEach(item -> {
            item.setContent(itemMap.get(item.getId()).getContent())
                    .setImgs(itemMap.get(item.getId()).getImgs()).setUpdateTime(now);
        });
        this.checkTaskItemMapper.updateBatch(taskItems);

        return "任务暂存成功!";
    }

    @Override
    public Long countCur(DmsCheckTaskDO checkTask) {

        LambdaQueryWrapper<DmsCheckTaskDO> wrapper = new LambdaQueryWrapper<DmsCheckTaskDO>()
                .eq(DmsCheckTaskDO::getEndFlag, 0);
        if (Objects.nonNull(checkTask.getExecutorId())) {
            wrapper.eq(DmsCheckTaskDO::getExecutorId, checkTask.getExecutorId());
        }
        return this.checkTaskMapper.selectCount(wrapper);
    }

    @Override
    public DmsCheckTaskDO getFirst(DmsCheckTaskPageReqVO pageReqVO) {
        LambdaQueryWrapper<DmsCheckTaskDO> wrapper = new LambdaQueryWrapper<DmsCheckTaskDO>().eq(DmsCheckTaskDO::getExecutorId, pageReqVO.getExecutorId())
                .eq(DmsCheckTaskDO::getEndFlag, 0).orderByAsc(DmsCheckTaskDO::getStartTime).last("limit 1");
        return this.checkTaskMapper.selectOne(wrapper);
    }

}
