package cn.iocoder.yudao.module.dms.service.device;

import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.common.util.object.BeanUtils;
import cn.iocoder.yudao.framework.security.core.util.SecurityFrameworkUtils;
import cn.iocoder.yudao.module.dms.controller.admin.device.vo.DmsDevicePageReqVO;
import cn.iocoder.yudao.module.dms.controller.admin.device.vo.DmsDeviceSaveReqVO;
import cn.iocoder.yudao.module.dms.dal.dataobject.device.DmsDeviceDO;
import cn.iocoder.yudao.module.dms.dal.pgsql.device.DmsDeviceMapper;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import org.springframework.stereotype.Service;
import org.springframework.validation.annotation.Validated;

import javax.annotation.Resource;
import java.util.Objects;

import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.dms.enums.ErrorCodeConstants.DEVICE_NOT_EXISTS;

/**
 * 设备管理 Service 实现类
 *
 * @author 芋道源码
 */
@Service
@Validated
public class DmsDeviceServiceImpl implements DmsDeviceService {

    @Resource
    private DmsDeviceMapper dmsDeviceMapper;

    @Override
    public Long createDevice(DmsDeviceSaveReqVO createReqVO) {
        // 插入
        DmsDeviceDO device = BeanUtils.toBean(createReqVO, DmsDeviceDO.class);
        device.setDeptId(SecurityFrameworkUtils.getLoginUserDeptId());
        dmsDeviceMapper.insert(device);
        // 返回
        return device.getId();
    }

    @Override
    public void updateDevice(DmsDeviceSaveReqVO updateReqVO) {
        // 校验存在
        validateDeviceExists(updateReqVO.getId());
        // 更新
        DmsDeviceDO updateObj = BeanUtils.toBean(updateReqVO, DmsDeviceDO.class);
        dmsDeviceMapper.updateById(updateObj);
    }

    @Override
    public void deleteDevice(Long id) {
        // 校验存在
        validateDeviceExists(id);
        // 删除
        dmsDeviceMapper.deleteById(id);
    }

    private void validateDeviceExists(Long id) {
        if (dmsDeviceMapper.selectById(id) == null) {
            throw exception(DEVICE_NOT_EXISTS);
        }
    }

    @Override
    public DmsDeviceDO getDevice(Long id) {
        return dmsDeviceMapper.selectById(id);
    }

    @Override
    public PageResult<DmsDeviceDO> getDevicePage(DmsDevicePageReqVO pageReqVO) {
        return dmsDeviceMapper.selectPage(pageReqVO);
    }

    @Override
    public Long location(DmsDeviceSaveReqVO saveReqVO) {
        DmsDeviceDO dbDevice = this.dmsDeviceMapper.selectById(saveReqVO.getId());
        dbDevice.setLat(saveReqVO.getLat()).setLng(saveReqVO.getLng());
        this.dmsDeviceMapper.updateById(dbDevice);
        return dbDevice.getId();
    }

    @Override
    public Long locationCount(DmsDeviceSaveReqVO device) {
        LambdaQueryWrapper<DmsDeviceDO> queryWrapper = new LambdaQueryWrapper<DmsDeviceDO>().isNull(DmsDeviceDO::getLng);
        if (Objects.nonNull(device.getManagerId())) {
            queryWrapper.eq(DmsDeviceDO::getManagerId, device.getManagerId());
        }
        return this.dmsDeviceMapper.selectCount(queryWrapper);
    }

}
