package cn.iocoder.yudao.module.dms.service.deviceattribute;

import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.common.util.object.BeanUtils;
import cn.iocoder.yudao.module.dms.controller.admin.deviceattribute.vo.DmsDeviceAttributePageReqVO;
import cn.iocoder.yudao.module.dms.controller.admin.deviceattribute.vo.DmsDeviceAttributeSaveReqVO;
import cn.iocoder.yudao.module.dms.dal.dataobject.deviceattribute.DmsDeviceAttributeDO;
import cn.iocoder.yudao.module.dms.dal.pgsql.deviceattribute.DmsDeviceAttributeMapper;
import org.springframework.stereotype.Service;
import org.springframework.validation.annotation.Validated;

import javax.annotation.Resource;

import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.dms.enums.ErrorCodeConstants.DEVICE_ATTRIBUTE_NOT_EXISTS;

/**
 * 设备属性 Service 实现类
 *
 * @author dzx
 */
@Service
@Validated
public class DmsDeviceAttributeServiceImpl implements DmsDeviceAttributeService {

    @Resource
    private DmsDeviceAttributeMapper deviceAttributeMapper;

    @Override
    public Long createDeviceAttribute(DmsDeviceAttributeSaveReqVO createReqVO) {
        // 插入
        DmsDeviceAttributeDO deviceAttribute = BeanUtils.toBean(createReqVO, DmsDeviceAttributeDO.class);
        deviceAttributeMapper.insert(deviceAttribute);
        // 返回
        return deviceAttribute.getId();
    }

    @Override
    public void updateDeviceAttribute(DmsDeviceAttributeSaveReqVO updateReqVO) {
        // 校验存在
        validateDeviceAttributeExists(updateReqVO.getId());
        // 更新
        DmsDeviceAttributeDO updateObj = BeanUtils.toBean(updateReqVO, DmsDeviceAttributeDO.class);
        deviceAttributeMapper.updateById(updateObj);
    }

    @Override
    public void deleteDeviceAttribute(Long id) {
        // 校验存在
        validateDeviceAttributeExists(id);
        // 删除
        deviceAttributeMapper.deleteById(id);
    }

    private void validateDeviceAttributeExists(Long id) {
        if (deviceAttributeMapper.selectById(id) == null) {
            throw exception(DEVICE_ATTRIBUTE_NOT_EXISTS);
        }
    }

    @Override
    public DmsDeviceAttributeDO getDeviceAttribute(Long id) {
        return deviceAttributeMapper.selectById(id);
    }

    @Override
    public PageResult<DmsDeviceAttributeDO> getDeviceAttributePage(DmsDeviceAttributePageReqVO pageReqVO) {
        return deviceAttributeMapper.selectPage(pageReqVO);
    }

}
