package cn.iocoder.yudao.module.dms.service.devicerepair;

import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.common.util.object.BeanUtils;
import cn.iocoder.yudao.module.dms.controller.admin.devicerepair.vo.DmsDeviceRepairPageReqVO;
import cn.iocoder.yudao.module.dms.controller.admin.devicerepair.vo.DmsDeviceRepairSaveReqVO;
import cn.iocoder.yudao.module.dms.dal.dataobject.devicerepair.DmsDeviceRepairDO;
import cn.iocoder.yudao.module.dms.dal.pgsql.devicerepair.DmsDeviceRepairMapper;
import org.springframework.stereotype.Service;
import org.springframework.validation.annotation.Validated;

import javax.annotation.Resource;

import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.dms.enums.ErrorCodeConstants.DEVICE_REPAIR_NOT_EXISTS;

/**
 * 设备报修 Service 实现类
 *
 * @author dzx
 */
@Service
@Validated
public class DmsDeviceRepairServiceImpl implements DmsDeviceRepairService {

    @Resource
    private DmsDeviceRepairMapper deviceRepairMapper;

    @Override
    public Long createDeviceRepair(DmsDeviceRepairSaveReqVO createReqVO) {
        // 插入
        DmsDeviceRepairDO deviceRepair = BeanUtils.toBean(createReqVO, DmsDeviceRepairDO.class);
        deviceRepairMapper.insert(deviceRepair);
        // 返回
        return deviceRepair.getId();
    }

    @Override
    public void updateDeviceRepair(DmsDeviceRepairSaveReqVO updateReqVO) {
        // 校验存在
        validateDeviceRepairExists(updateReqVO.getId());
        // 更新
        DmsDeviceRepairDO updateObj = BeanUtils.toBean(updateReqVO, DmsDeviceRepairDO.class);
        deviceRepairMapper.updateById(updateObj);
    }

    @Override
    public void deleteDeviceRepair(Long id) {
        // 校验存在
        validateDeviceRepairExists(id);
        // 删除
        deviceRepairMapper.deleteById(id);
    }

    private void validateDeviceRepairExists(Long id) {
        if (deviceRepairMapper.selectById(id) == null) {
            throw exception(DEVICE_REPAIR_NOT_EXISTS);
        }
    }

    @Override
    public DmsDeviceRepairDO getDeviceRepair(Long id) {
        return deviceRepairMapper.selectById(id);
    }

    @Override
    public PageResult<DmsDeviceRepairDO> getDeviceRepairPage(DmsDeviceRepairPageReqVO pageReqVO) {
        return deviceRepairMapper.selectPage(pageReqVO);
    }

}
