package cn.iocoder.yudao.module.dms.service.relationorg;

import cn.iocoder.yudao.framework.common.pojo.PageParam;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.common.util.object.BeanUtils;
import cn.iocoder.yudao.framework.security.core.util.SecurityFrameworkUtils;
import cn.iocoder.yudao.module.dms.controller.admin.relationorg.vo.DmsRelationOrgPageReqVO;
import cn.iocoder.yudao.module.dms.controller.admin.relationorg.vo.DmsRelationOrgSaveReqVO;
import cn.iocoder.yudao.module.dms.dal.dataobject.relationorg.DmsRelationContractDO;
import cn.iocoder.yudao.module.dms.dal.dataobject.relationorg.DmsRelationOrgDO;
import cn.iocoder.yudao.module.dms.dal.pgsql.relationorg.DmsRelationContractMapper;
import cn.iocoder.yudao.module.dms.dal.pgsql.relationorg.DmsRelationOrgMapper;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.validation.annotation.Validated;

import javax.annotation.Resource;
import java.util.List;

import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.dms.enums.ErrorCodeConstants.RELATION_CONTRACT_NOT_EXISTS;
import static cn.iocoder.yudao.module.dms.enums.ErrorCodeConstants.RELATION_ORG_NOT_EXISTS;

/**
 * 往来单位 Service 实现类
 *
 * @author dzx
 */
@Service
@Validated
public class DmsRelationOrgServiceImpl implements DmsRelationOrgService {

    @Resource
    private DmsRelationOrgMapper relationOrgMapper;
    @Resource
    private DmsRelationContractMapper relationContractMapper;

    @Override
    public Long createRelationOrg(DmsRelationOrgSaveReqVO createReqVO) {
        // 插入
        DmsRelationOrgDO relationOrg = BeanUtils.toBean(createReqVO, DmsRelationOrgDO.class);
        relationOrg.setDeptId(SecurityFrameworkUtils.getLoginUserDeptId());
        relationOrgMapper.insert(relationOrg);
        // 返回
        return relationOrg.getId();
    }

    @Override
    public void updateRelationOrg(DmsRelationOrgSaveReqVO updateReqVO) {
        // 校验存在
        validateRelationOrgExists(updateReqVO.getId());
        // 更新
        DmsRelationOrgDO updateObj = BeanUtils.toBean(updateReqVO, DmsRelationOrgDO.class);
        relationOrgMapper.updateById(updateObj);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void deleteRelationOrg(Long id) {
        // 校验存在
        validateRelationOrgExists(id);
        // 删除
        relationOrgMapper.deleteById(id);

        // 删除子表
        deleteRelationContractByOrgId(id);
    }

    private void validateRelationOrgExists(Long id) {
        if (relationOrgMapper.selectById(id) == null) {
            throw exception(RELATION_ORG_NOT_EXISTS);
        }
    }

    @Override
    public DmsRelationOrgDO getRelationOrg(Long id) {
        return relationOrgMapper.selectById(id);
    }

    @Override
    public PageResult<DmsRelationOrgDO> getRelationOrgPage(DmsRelationOrgPageReqVO pageReqVO) {
        return relationOrgMapper.selectPage(pageReqVO);
    }

    /**
     * 获得往来单位精简列表
     *
     * @return
     */
    @Override
    public List<DmsRelationOrgDO> getSimpleRelationOrgList() {
        return relationOrgMapper.selectSimpleRelationOrgList();
    }

    // ==================== 子表（往来单位联系人） ====================

    @Override
    public PageResult<DmsRelationContractDO> getRelationContractPage(PageParam pageReqVO, Long orgId) {
        return relationContractMapper.selectPage(pageReqVO, orgId);
    }

    @Override
    public Long createRelationContract(DmsRelationContractDO relationContract) {
        relationContractMapper.insert(relationContract);
        return relationContract.getId();
    }

    @Override
    public void updateRelationContract(DmsRelationContractDO relationContract) {
        // 校验存在
        validateRelationContractExists(relationContract.getId());
        // 更新
        relationContractMapper.updateById(relationContract);
    }

    @Override
    public void deleteRelationContract(Long id) {
        // 校验存在
        validateRelationContractExists(id);
        // 删除
        relationContractMapper.deleteById(id);
    }

    @Override
    public DmsRelationContractDO getRelationContract(Long id) {
        return relationContractMapper.selectById(id);
    }

    private void validateRelationContractExists(Long id) {
        if (relationContractMapper.selectById(id) == null) {
            throw exception(RELATION_CONTRACT_NOT_EXISTS);
        }
    }

    private void deleteRelationContractByOrgId(Long orgId) {
        relationContractMapper.deleteByOrgId(orgId);
    }

}
